"""Python async interface to Loadstar Sensors USB devices."""
# This file is generated automatically from metadata
# File edits may be overwritten!

__version__ = '1.1.0'
__description__ = 'Python async interface to Loadstar Sensors USB devices.'
__license__ = 'BSD-3-Clause'
__url__ = 'https://github.com/janelia-pypi/loadstar_sensors_interface_python'
__author__ = 'Peter Polidoro'
__email__ = 'peter@polidoro.io'
__copyright__ = '2023 Howard Hughes Medical Institute'
