"""Python async interface to Loadstar Sensors USB devices."""
# This file is generated automatically from metadata
# File edits may be overwritten!
from loadstar_sensors_interface.__about__ import (
    __author__,
    __copyright__,
    __email__,
    __license__,
    __description__,
    __url__,
    __version__)

from .loadstar_sensors_interface import LoadstarSensorsInterface
